@@ -27,7 +27,7 @@ from .common import (_ensure_object, is_bool, is_integer, is_float,
 from .dtypes import ExtensionDtype, DatetimeTZDtype, PeriodDtype
 from .generic import (ABCDatetimeIndex, ABCPeriodIndex,
                       ABCSeries)
-from .missing import isnull, notnull
+from .missing import isna, notna
 from .inference import is_list_like
 
 _int8_max = np.iinfo(np.int8).max
@@ -110,9 +110,7 @@ def maybe_downcast_to_dtype(result, dtype):
                     np.prod(result.shape)):
                 return result
 
-        if issubclass(dtype.type, np.floating):
-            return result.astype(dtype)
-        elif is_bool_dtype(dtype) or is_integer_dtype(dtype):
+        if is_bool_dtype(dtype) or is_integer_dtype(dtype):
 
             # if we don't have any elements, just astype it
             if not np.prod(result.shape):
@@ -123,7 +121,7 @@ def maybe_downcast_to_dtype(result, dtype):
             arr = np.array([r[0]])
 
             # if we have any nulls, then we are done
-            if (isnull(arr).any() or
+            if (isna(arr).any() or
                     not np.allclose(arr, trans(arr).astype(dtype), rtol=0)):
                 return result
 
@@ -133,7 +131,7 @@ def maybe_downcast_to_dtype(result, dtype):
                 return result
 
             if (issubclass(result.dtype.type, (np.object_, np.number)) and
-                    notnull(result).all()):
+                    notna(result).all()):
                 new_result = trans(result).astype(dtype)
                 try:
                     if np.allclose(new_result, result, rtol=0):
@@ -144,6 +142,9 @@ def maybe_downcast_to_dtype(result, dtype):
                     # hit here
                     if (new_result == result).all():
                         return new_result
+        elif (issubclass(dtype.type, np.floating) and
+                not is_bool_dtype(result.dtype)):
+            return result.astype(dtype)
 
         # a datetimelike
         # GH12821, iNaT is casted to float
@@ -190,7 +191,7 @@ def maybe_upcast_putmask(result, mask, other):
         #   integer or integer array -> date-like array
         if is_datetimelike(result.dtype):
             if is_scalar(other):
-                if isnull(other):
+                if isna(other):
                     other = result.dtype.type('nat')
                 elif is_integer(other):
                     other = np.array(other, dtype=result.dtype)
@@ -231,13 +232,13 @@ def maybe_upcast_putmask(result, mask, other):
             # and its nan and we are changing some values
             if (is_scalar(other) or
                     (isinstance(other, np.ndarray) and other.ndim < 1)):
-                if isnull(other):
+                if isna(other):
                     return changeit()
 
             # we have an ndarray and the masking has nans in it
             else:
 
-                if isnull(other[mask]).any():
+                if isna(other[mask]).any():
                     return changeit()
 
         try:
@@ -267,12 +268,12 @@ def maybe_promote(dtype, fill_value=np.nan):
         # for now: refuse to upcast datetime64
         # (this is because datetime64 will not implicitly upconvert
         #  to object correctly as of numpy 1.6.1)
-        if isnull(fill_value):
+        if isna(fill_value):
             fill_value = iNaT
         else:
             if issubclass(dtype.type, np.datetime64):
                 try:
-                    fill_value = lib.Timestamp(fill_value).value
+                    fill_value = tslib.Timestamp(fill_value).value
                 except:
                     # the proper thing to do here would probably be to upcast
                     # to object (but numpy 1.6.1 doesn't do this properly)
@@ -286,7 +287,7 @@ def maybe_promote(dtype, fill_value=np.nan):
             else:
                 fill_value = iNaT
     elif is_datetimetz(dtype):
-        if isnull(fill_value):
+        if isna(fill_value):
             fill_value = iNaT
     elif is_float(fill_value):
         if issubclass(dtype.type, np.bool_):
@@ -333,6 +334,23 @@ def maybe_promote(dtype, fill_value=np.nan):
     return dtype, fill_value
 
 
+def infer_dtype_from(val, pandas_dtype=False):
+    """
+    interpret the dtype from a scalar or array. This is a convenience
+    routines to infer dtype from a scalar or an array
+
+    Parameters
+    ----------
+    pandas_dtype : bool, default False
+        whether to infer dtype including pandas extension types.
+        If False, scalar/array belongs to pandas extension types is inferred as
+        object
+    """
+    if is_scalar(val):
+        return infer_dtype_from_scalar(val, pandas_dtype=pandas_dtype)
+    return infer_dtype_from_array(val, pandas_dtype=pandas_dtype)
+
+
 def infer_dtype_from_scalar(val, pandas_dtype=False):
     """
     interpret the dtype from a scalar
@@ -349,9 +367,9 @@ def infer_dtype_from_scalar(val, pandas_dtype=False):
 
     # a 1-element ndarray
     if isinstance(val, np.ndarray):
+        msg = "invalid ndarray passed to _infer_dtype_from_scalar"
         if val.ndim != 0:
-            raise ValueError(
-                "invalid ndarray passed to _infer_dtype_from_scalar")
+            raise ValueError(msg)
 
         dtype = val.dtype
         val = val.item()
@@ -408,24 +426,31 @@ def infer_dtype_from_scalar(val, pandas_dtype=False):
     return dtype, val
 
 
-def infer_dtype_from_array(arr):
+def infer_dtype_from_array(arr, pandas_dtype=False):
     """
     infer the dtype from a scalar or array
 
     Parameters
     ----------
     arr : scalar or array
+    pandas_dtype : bool, default False
+        whether to infer dtype including pandas extension types.
+        If False, array belongs to pandas extension types
+        is inferred as object
 
     Returns
     -------
-    tuple (numpy-compat dtype, array)
+    tuple (numpy-compat/pandas-compat dtype, array)
 
     Notes
     -----
-    These infer to numpy dtypes exactly
-    with the exception that mixed / object dtypes
+    if pandas_dtype=False. these infer to numpy dtypes
+    exactly with the exception that mixed / object dtypes
     are not coerced by stringifying or conversion
 
+    if pandas_dtype=True. datetime64tz-aware/categorical
+    types will retain there character.
+
     Examples
     --------
     >>> np.asarray([1, '1'])
@@ -442,6 +467,12 @@ def infer_dtype_from_array(arr):
     if not is_list_like(arr):
         arr = [arr]
 
+    if pandas_dtype and is_extension_type(arr):
+        return arr.dtype, arr
+
+    elif isinstance(arr, ABCSeries):
+        return arr.dtype, np.asarray(arr)
+
     # don't force numpy coerce with nan's
     inferred = lib.infer_dtype(arr)
     if inferred in ['string', 'bytes', 'unicode',
@@ -452,6 +483,39 @@ def infer_dtype_from_array(arr):
     return arr.dtype, arr
 
 
+def maybe_infer_dtype_type(element):
+    """Try to infer an object's dtype, for use in arithmetic ops
+
+    Uses `element.dtype` if that's available.
+    Objects implementing the iterator protocol are cast to a NumPy array,
+    and from there the array's type is used.
+
+    Parameters
+    ----------
+    element : object
+        Possibly has a `.dtype` attribute, and possibly the iterator
+        protocol.
+
+    Returns
+    -------
+    tipo : type
+
+    Examples
+    --------
+    >>> from collections import namedtuple
+    >>> Foo = namedtuple("Foo", "dtype")
+    >>> maybe_infer_dtype_type(Foo(np.dtype("i8")))
+    numpy.int64
+    """
+    tipo = None
+    if hasattr(element, 'dtype'):
+        tipo = element.dtype
+    elif is_list_like(element):
+        element = np.asarray(element)
+        tipo = element.dtype
+    return tipo
+
+
 def maybe_upcast(values, fill_value=np.nan, dtype=None, copy=False):
     """ provide explict type promotion and coercion
 
@@ -485,7 +549,8 @@ def maybe_cast_item(obj, item, dtype):
         if dtype in (np.object_, np.bool_):
             obj[item] = chunk.astype(np.object_)
         elif not issubclass(dtype, (np.integer, np.bool_)):  # pragma: no cover
-            raise ValueError("Unexpected dtype encountered: %s" % dtype)
+            raise ValueError("Unexpected dtype encountered: {dtype}"
+                             .format(dtype=dtype))
 
 
 def invalidate_string_dtypes(dtype_set):
@@ -549,10 +614,10 @@ def coerce_to_dtypes(result, dtypes):
 
     def conv(r, dtype):
         try:
-            if isnull(r):
+            if isna(r):
                 pass
             elif dtype == _NS_DTYPE:
-                r = lib.Timestamp(r)
+                r = tslib.Timestamp(r)
             elif dtype == _TD_DTYPE:
                 r = _coerce_scalar_to_timedelta_type(r)
             elif dtype == np.bool_:
@@ -589,8 +654,9 @@ def astype_nansafe(arr, dtype, copy=True):
         elif dtype == np.int64:
             return arr.view(dtype)
         elif dtype != _NS_DTYPE:
-            raise TypeError("cannot astype a datetimelike from [%s] to [%s]" %
-                            (arr.dtype, dtype))
+            raise TypeError("cannot astype a datetimelike from [{from_dtype}] "
+                            "to [{to_dtype}]".format(from_dtype=arr.dtype,
+                                                     to_dtype=dtype))
         return arr.astype(_NS_DTYPE)
     elif is_timedelta64_dtype(arr):
         if dtype == np.int64:
@@ -604,13 +670,14 @@ def astype_nansafe(arr, dtype, copy=True):
 
             # allow frequency conversions
             if dtype.kind == 'm':
-                mask = isnull(arr)
+                mask = isna(arr)
                 result = arr.astype(dtype).astype(np.float64)
                 result[mask] = np.nan
                 return result
 
-            raise TypeError("cannot astype a timedelta from [%s] to [%s]" %
-                            (arr.dtype, dtype))
+            raise TypeError("cannot astype a timedelta from [{from_dtype}] "
+                            "to [{to_dtype}]".format(from_dtype=arr.dtype,
+                                                     to_dtype=dtype))
 
         return arr.astype(_TD_DTYPE)
     elif (np.issubdtype(arr.dtype, np.floating) and
@@ -656,7 +723,7 @@ def maybe_convert_objects(values, convert_dates=True, convert_numeric=True,
                 values, 'M8[ns]', errors='coerce')
 
             # if we are all nans then leave me alone
-            if not isnull(new_values).all():
+            if not isna(new_values).all():
                 values = new_values
 
         else:
@@ -668,10 +735,10 @@ def maybe_convert_objects(values, convert_dates=True, convert_numeric=True,
 
         if convert_timedeltas == 'coerce':
             from pandas.core.tools.timedeltas import to_timedelta
-            new_values = to_timedelta(values, coerce=True)
+            new_values = to_timedelta(values, errors='coerce')
 
             # if we are all nans then leave me alone
-            if not isnull(new_values).all():
+            if not isna(new_values).all():
                 values = new_values
 
         else:
@@ -686,7 +753,7 @@ def maybe_convert_objects(values, convert_dates=True, convert_numeric=True,
                                                        coerce_numeric=True)
 
                 # if we are all nans then leave me alone
-                if not isnull(new_values).all():
+                if not isna(new_values).all():
                     values = new_values
 
             except:
@@ -748,7 +815,7 @@ def soft_convert_objects(values, datetime=True, numeric=True, timedelta=True,
             converted = lib.maybe_convert_numeric(values, set(),
                                                   coerce_numeric=True)
             # If all NaNs, then do not-alter
-            values = converted if not isnull(converted).all() else values
+            values = converted if not isna(converted).all() else values
             values = values.copy() if copy else values
         except:
             pass
@@ -850,7 +917,7 @@ def maybe_infer_to_datetimelike(value, convert_dates=False):
     elif inferred_type == 'nat':
 
         # if all NaT, return as datetime
-        if isnull(v).all():
+        if isna(v).all():
             value = try_datetime(v)
         else:
 
@@ -895,13 +962,13 @@ def maybe_cast_to_datetime(value, dtype, errors='raise'):
                     dtype = _NS_DTYPE
                 else:
                     raise TypeError("cannot convert datetimelike to "
-                                    "dtype [%s]" % dtype)
+                                    "dtype [{dtype}]".format(dtype=dtype))
             elif is_datetime64tz:
 
                 # our NaT doesn't support tz's
                 # this will coerce to DatetimeIndex with
                 # a matching dtype below
-                if is_scalar(value) and isnull(value):
+                if is_scalar(value) and isna(value):
                     value = [value]
 
             elif is_timedelta64 and not is_dtype_equal(dtype, _TD_DTYPE):
@@ -912,10 +979,10 @@ def maybe_cast_to_datetime(value, dtype, errors='raise'):
                     dtype = _TD_DTYPE
                 else:
                     raise TypeError("cannot convert timedeltalike to "
-                                    "dtype [%s]" % dtype)
+                                    "dtype [{dtype}]".format(dtype=dtype))
 
             if is_scalar(value):
-                if value == iNaT or isnull(value):
+                if value == iNaT or isna(value):
                     value = iNaT
             else:
                 value = np.array(value, copy=False)
@@ -951,7 +1018,8 @@ def maybe_cast_to_datetime(value, dtype, errors='raise'):
                 return tslib.ints_to_pydatetime(ints)
 
             # we have a non-castable dtype that was passed
-            raise TypeError('Cannot cast datetime64 to %s' % dtype)
+            raise TypeError('Cannot cast datetime64 to {dtype}'
+                            .format(dtype=dtype))
 
     else:
 
@@ -1026,3 +1094,31 @@ def find_common_type(types):
             return np.object
 
     return np.find_common_type(types, [])
+
+
+def cast_scalar_to_array(shape, value, dtype=None):
+    """
+    create np.ndarray of specified shape and dtype, filled with values
+
+    Parameters
+    ----------
+    shape : tuple
+    value : scalar value
+    dtype : np.dtype, optional
+        dtype to coerce
+
+    Returns
+    -------
+    ndarray of shape, filled with value, of specified / inferred dtype
+
+    """
+
+    if dtype is None:
+        dtype, fill_value = infer_dtype_from_scalar(value)
+    else:
+        fill_value = value
+
+    values = np.empty(shape, dtype=dtype)
+    values.fill(fill_value)
+
+    return values
